# OBJBOT - Framework to program bots !!
#
# obj/irc/dcc.py (direct client to client bot)
#
# Copyright 2017,2018 B.H.J Thate
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice don't have to be included.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# 01-03-2018 As the creator of this file, I disclaim all rights on  this file. 
#
# Bart Thate
# Heerhugowaard
# The Netherlands

""" direct client to client bot. """

## imports

from obj.base import Object
from obj.fleet import Fleet
from obj.loader import Loader
from obj.select import Select


import io
import logging
import os
import socket
import sys

## vars

fleet = Fleet()
loader = Loader()

## classes

class Event(Object):

    """ DCC bot event. """

    def raw(self, txt):
        """ use fleet to output to bot. """
        fleet.say_id(self.id, self.channel, txt, "normal")

class DCC(Select):

    """ do communication over a file descriptor. """

    cc = ""

    def event(self):
        """ create a DCC event, """
        txt = self._sockfile.readline()
        if len(txt) == 0:
            return
        txt = txt.strip()
        logging.warn("< %s" % txt)
        e = Event()
        e.id = self.id
        e._loader = loader
        e.cc = self.cc
        e.txt = txt
        e.channel = ""
        e.origin = self.origin or "root@dcc"
        return e

    def say(self, channel, txt, type="normal"):
        """ return text to the DCC socket. """
        try:
            self._sockfile.write(txt)
            self._sockfile.write("\n")
            self._sockfile.flush()
        except:
            pass

    def start(self, *args, **kwargs):
        """ start the DCC bot. """
        self._target = args[0]
        self._resume.fd = self._target.fileno()
        self._sockfile = self._target.makefile("rw")
        super().start(self, *args, **kwargs)
        self.register_fd(self._resume.fd)

